//-----------------------------------------------------------------------------------------//
// BigBlur.fx
//
// Credits : khaver & schrauber
//-----------------------------------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Blur";
   string Category    = "Stylize";
   string Notes       = "High quality blur using a rotating Poisson distribution";
   bool CanSize       = false;
> = 0;

//-----------------------------------------------------------------------------------------//
// Inputs & Samplers
//-----------------------------------------------------------------------------------------//

texture Input;
texture Blur : RenderColorTarget;

sampler FgdSampler = sampler_state
{
   Texture   = <Input>;
   AddressU  = Mirror;
   AddressV  = Mirror;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
};

sampler render_Sampler = sampler_state
{
   Texture   = <Blur>;
   AddressU  = Mirror;
   AddressV  = Mirror;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;
};

//-----------------------------------------------------------------------------------------//
// Parameters
//-----------------------------------------------------------------------------------------//

float Size
<
   string Description = "Radius";
   float MinVal       = 0.0;
   float MaxVal       = 1.0;
> = 0.1;

//-----------------------------------------------------------------------------------------//
// Definitions and declarations
//-----------------------------------------------------------------------------------------//

float _OutputAspectRatio;
float _OutputWidth;

//-----------------------------------------------------------------------------------------//
// Shaders
//-----------------------------------------------------------------------------------------//

float4 PS_bigBlur_b (float2 uv : TEXCOORD1, uniform sampler blurSampler, uniform int run) : COLOR
{
   float blur = Size *_OutputWidth * 0.1;
   float2 pixelSize = float2 (1.0, _OutputAspectRatio) / _OutputWidth;

   float2 halfpix = pixelSize / 2.0;

   float2 coord, sample;

   float4 cOut = tex2D (blurSampler, uv - halfpix);

   for (int tap = 0; tap < 12; tap++) {
      sincos ((tap * 30) + (run * 5), sample.y, sample.x);   // 30 is the rotation angle between the 12 samples, 5 is the additional rotation of all rotation angles per pass (30 / 6 pass = 5).
      coord = uv + (halfpix * sample * blur);
      cOut += tex2D (blurSampler, coord);
   }

   cOut /= 13.0;

   return cOut;
}

//-----------------------------------------------------------------------------------------//
// Techniques
//-----------------------------------------------------------------------------------------//
technique Big_blur
{
    pass P_1  < string Script = "RenderColorTarget0 = Blur;"; >  { PixelShader = compile PROFILE  PS_bigBlur_b (FgdSampler,     0); }
    pass P_2  < string Script = "RenderColorTarget0 = Blur;"; >  { PixelShader = compile PROFILE  PS_bigBlur_b (render_Sampler, 1); }
    pass P_3  < string Script = "RenderColorTarget0 = Blur;"; >  { PixelShader = compile PROFILE  PS_bigBlur_b (render_Sampler, 2); }
    pass P_4  < string Script = "RenderColorTarget0 = Blur;"; >  { PixelShader = compile PROFILE  PS_bigBlur_b (render_Sampler, 3); }
    pass P_5  < string Script = "RenderColorTarget0 = Blur;"; >  { PixelShader = compile PROFILE  PS_bigBlur_b (render_Sampler, 4); }
    pass P_6                                                     { PixelShader = compile PROFILE  PS_bigBlur_b (render_Sampler, 5); }
}
