//--------------------------------------------------------------//
// Colour Tint
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Colour Tint";
   string Category    = "Colour Effects";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float4 TintColour
<
   string Description = "Tint Colour";
> = { 1.0, 1.0, 1.0, 1.0 };

float Amount
<
   string Description = "Amount";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 0.5;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( Input, InputSampler );

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = ReadPixel( InputSampler, xy1 );
   float lum = ( src.r * 0.3 ) + ( src.g * 0.59 ) + ( src.b * 0.11 );

   float4 tintedPixel = float4( lum + TintColour.r - 0.5,
                                lum + TintColour.g - 0.5,
                                lum + TintColour.b - 0.5,
                                1.0 );

   float4 ret = lerp( src, tintedPixel, Amount * src.a );
   ret.a = src.a;

   return ret;
}


//--------------------------------------------------------------//
// Technique Section for Image Processing Effects
//--------------------------------------------------------------//
technique ColourTint { pass Single_Pass { PixelShader = compile PROFILE ps_main(); } }
