//--------------------------------------------------------------//
// curves.fx
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Curves";
   string Category    = "Colour";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( Input, InputSampler );

texture _map;

sampler MapSampler = sampler_state
{
   Texture  = <_map>;
   AddressU = ClampToEdge;
   AddressV = ClampToEdge;
};

float _offset = 0.5 / 256.0;
float _scale   = 255.0 / 256.0;

//---------------------------------------------------------------------------------------//
// Logic
//---------------------------------------------------------------------------------------//
float2 makeLUTCoordinate( float componentValue )
{
   return float2( ( componentValue * _scale ) + _offset, 0.5 );
}
//---------------------------------------------------------------------------------------//
float4 rgbMain( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source  = ReadPixel( InputSampler,  xy1 );
   float4 mappedR = tex2D( MapSampler, makeLUTCoordinate( source.r ) );
   float4 mappedG = tex2D( MapSampler, makeLUTCoordinate( source.g ) );
   float4 mappedB = tex2D( MapSampler, makeLUTCoordinate( source.b ) );

   return float4( mappedR.r, mappedG.g, mappedB.b, source.a );
}
//---------------------------------------------------------------------------------------//
float4 rgblMain( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source  = ReadPixel( InputSampler,  xy1 );
   float4 mappedR = tex2D( MapSampler, makeLUTCoordinate( source.r ) );
   float4 mappedG = tex2D( MapSampler, makeLUTCoordinate( source.g ) );
   float4 mappedB = tex2D( MapSampler, makeLUTCoordinate( source.b ) );

   // Calc pixel luminance
   float lum = ( source.r * 0.3 ) + ( source.g * 0.59 ) + ( source.b * 0.11 );

   // Get the mapped luminance value
   float4 mappedLum = tex2D( MapSampler,  makeLUTCoordinate( lum )  );

   // Recalc RGB using the new luminance value
   float lumMultiplier = 1;

   if ( lum != 0 )
      lumMultiplier = mappedLum.a / lum;

   return float4( mappedR.r * lumMultiplier, mappedG.g * lumMultiplier, mappedB.b * lumMultiplier, source.a );
}

technique CurvesRGB  { pass One { PixelShader = compile PROFILE rgbMain(); } }
technique CurvesRGBL { pass One { PixelShader = compile PROFILE rgblMain(); } }

