//--------------------------------------------------------------//
// Greyscale
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Greyscale";
   string Category    = "Colour Effects";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Red
<
   string Description = "Red";
   string Group = "Channel Weights";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.3;

float Green
<
   string Description = "Green";
   string Group = "Channel Weights";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.59;

float Blue
<
   string Description = "Blue";
   string Group = "Channel Weights";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.11;

float RedContrast
<
   string Description = "Red";
   string Group = "Channel Contrasts";
   float MinVal = -1.00;
   float MaxVal = 4.00;
> = 1.0;

float GreenContrast
<
   string Description = "Green";
   string Group = "Channel Contrasts";
   float MinVal = -1.00;
   float MaxVal = 4.00;
> = 1.0;

float BlueContrast
<
   string Description = "Blue";
   string Group = "Channel Contrasts";
   float MinVal = -1.00;
   float MaxVal = 4.00;
> = 1.0;


//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( Input, InputSampler );

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float3 mult     = float3( Red, Green, Blue );
   float3 contrast = float3( RedContrast, GreenContrast, BlueContrast );
   float3 oneHalf  = float3( 0.5, 0.5, 0.5 );

   float4 source   = ReadPixel( InputSampler, xy1 );
   float3 rgb      = source.rgb;
   float3 tmp      = ( ( ( source - oneHalf ) * contrast ) + oneHalf ) * mult;
   float  level    = tmp.r + tmp.g + tmp.b;

   float4 greyPixel = float4( level, level, level, 1.0 );

   float4 ret = lerp( source, greyPixel, source.a );
   ret.a = source.a;

   return ret;
}

//--------------------------------------------------------------//
// Technique Section for Image Processing Effects
//--------------------------------------------------------------//
technique Effect { pass Single_Pass { PixelShader = compile PROFILE ps_main(); } }
