//--------------------------------------------------------------//
// lut3d.fx
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "3D LUT";
   string Category    = "Colour";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( Input, InputSampler );

texture _lut;

sampler3D CubeSampler = sampler_state
{
   Texture   = <_lut>;
   MagFilter = Linear;
   MinFilter = Linear;
   AddressU  = ClampToEdge;
   AddressV  = ClampToEdge;
   AddressW  = ClampToEdge;
};

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Amount
<
   string Description = "Amount";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 1.0;

float _scale  = 1.0f;
float _offset = 0.0f;

//--------------------------------------------------------------//
float4 applyCube( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 sourcePixel = ReadPixel( InputSampler, xy );
   float4 cubedPixel = tex3D( CubeSampler, ( sourcePixel.rgb * _scale ) + _offset );

   float4 ret = lerp( sourcePixel, cubedPixel, Amount );
   ret.a = sourcePixel.a;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique T1 { pass P1 { PixelShader = compile PROFILE applyCube(); } }
