//--------------------------------------------------------------//
// Mosaic
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Mosaic";
   string Category    = "Stylize";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Size
<
   string Description = "Size";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.01;

float _OutputAspectRatio;
float _InputWidthNormalised = 1.0;
float _InputHeightNormalised = 1.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( Input, InputSampler );

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret;

   if ( IsOutOfBounds( xy1 ) )
   {
      ret = kTransparentBlack;
   }
   else
   {
      float4 source = tex2D( InputSampler, xy1 );

      if ( ( xy1.x >= 0 ) && ( xy1.x <= 1.0 ) )
      {
         if ( Size != 0.0 )
         {
            float xSize = Size;
            float ySize = xSize * _OutputAspectRatio;

            float x = round( ( xy1.x - ( _InputWidthNormalised / 2 ) ) / xSize ) * xSize;
            float y = round( ( xy1.y - ( _InputHeightNormalised / 2 ) ) / ySize ) * ySize;

            ret = tex2D( InputSampler, float2( x + ( _InputWidthNormalised / 2 ), y + ( _InputHeightNormalised / 2 ) ) );

            ret = lerp( source, ret, source.a );
         }
         else
            ret = source;
      }
   }

   return ret;
}

technique Mosaic { pass Single_Pass { PixelShader = compile PROFILE ps_main(); } }
