//--------------------------------------------------------------//
// Null
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Null";
   string Category    = "Null";
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Y;
texture Cr;
texture Cb;

//?? single channel?
//Texture2D<float> Y;
//Texture2D<float> Cr;
//Texture2D<float> Cb;

//--------------------------------------------------------------//
// Samplers
//--------------------------------------------------------------//
sampler YSampler  = sampler_state
{
   Texture = <Y>;
   MINFILTER = POINT;
   MIPFILTER = POINT;
   MAGFILTER = POINT;
};
sampler CrSampler = sampler_state
{
   Texture = <Cr>;
   MINFILTER = POINT;
   MIPFILTER = POINT;
   MAGFILTER = POINT;
};
sampler CbSampler = sampler_state
{
   Texture = <Cb>;
   MINFILTER = POINT;
   MIPFILTER = POINT;
   MAGFILTER = POINT;
};

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// naming c.f. YCrCb422_Video_BT601_To_RGB_Unscaled_Point

   // R = Y + 1.402     * Cr
   // G = Y - 0.7141363 * Cr - 0.3441363 * Cb
   // B = Y                  + 1.772     * Cb

   // Substituting constants for BT-709:

   // R = Y + 1.5748031 * Cr
   // G = Y - 0.4681252 * Cr - 0.1872927 * Cb
   // B = Y                  + 1.8552876 * Cb

//The above assume that the input ranges are Y 0..1, Cr and Cb -0.5..+0.5. This does not correspond
//to the values normally used which will, in 10-bit land, be 64-940 for Y, 64-963 for Cr and Cb.

float4 PlanarYCrCb_Video_BT601_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2, float2 xy3 : TEXCOORD3 ) : COLOR
{
   float Y = tex2D( YSampler, xy1 ).a;
   float Cr = tex2D( CrSampler, xy2 ).a - 0.5;
   float Cb = tex2D( CbSampler, xy3 ).a - 0.5;

   float4 ret;

   ret.b = Y + 1.5748031 * Cr;
   ret.g = Y - 0.4681252 * Cr - 0.1872927 * Cb;
   ret.r = Y + 1.8552876 * Cb;

   ret.a = 1;

   return ret;
}

float4 PlanarYCrCb_Unscaled_BT601_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2, float2 xy3 : TEXCOORD3 ) : COLOR
{
   float Y = tex2D( YSampler, xy1 ).a;
   float Cr = tex2D( CrSampler, xy2 ).a - 0.5;
   float Cb = tex2D( CbSampler, xy3 ).a - 0.5;

   float4 ret;

   ret.b = Y + 1.402 * Cr;
   ret.g = Y - 0.7141363 * Cr - 0.3441363 * Cb;
   ret.r = Y + 1.772 * Cb;

   ret.a = 1;

   return ret;
}

float4 PlanarYCrCb_Video_BT709_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2, float2 xy3 : TEXCOORD3 ) : COLOR
{
   float Y  = tex2D( YSampler, xy1 ).a;
   float Cr = tex2D( CrSampler, xy2 ).a - 0.5;
   float Cb = tex2D( CbSampler, xy3 ).a - 0.5;

   float4 ret;

   ret.b = Y + 1.5748031 * Cr;
   ret.g = Y - 0.4681252 * Cr - 0.1872927 * Cb;
   ret.r = Y                  + 1.8552876 * Cb;

   ret.a = 1;

   return ret;
}

float4 PlanarYCrCb_Unscaled_BT709_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2, float2 xy3 : TEXCOORD3 ) : COLOR
{
   float Y  = tex2D( YSampler, xy1 ).a;
   float Cr = tex2D( CrSampler, xy2 ).a - 0.5;
   float Cb = tex2D( CbSampler, xy3 ).a - 0.5;

   float4 ret;

   ret.b = Y + 1.5748031 * Cr;
   ret.g = Y - 0.4681252 * Cr - 0.1872927 * Cb;
   ret.r = Y                  + 1.8552876 * Cb;

   ret.a = 1;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique PlanarYCrCb_Video_BT601 { pass P1 { PixelShader = compile PROFILE PlanarYCrCb_Video_BT601_main(); } }
technique PlanarYCrCb_Video_BT709 { pass P1 { PixelShader = compile PROFILE PlanarYCrCb_Video_BT709_main(); } }
technique PlanarYCrCb_Unscaled_BT601 { pass P1 { PixelShader = compile PROFILE PlanarYCrCb_Unscaled_BT601_main(); } }
technique PlanarYCrCb_Unscaled_BT709 { pass P1 { PixelShader = compile PROFILE PlanarYCrCb_Unscaled_BT709_main(); } }
