//--------------------------------------------------------------//
// Push.fx
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Push";
   string Category    = "Mixes";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
int SetTechnique
<
   string Description = "Type";
   string Enum = "Push Right,Push Down,Push Left,Push Up";
> = 0;

float Progress
<
   string Description = "Progress";
   float MinVal = 0.0;
   float MaxVal = 1.0;
   float KF0    = 0.0;
   float KF1    = 1.0;
> = 0.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( fg, FgSampler );
DeclareInput( bg, BgSampler );

float _bgNormWidth = 1.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 PushRight( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.x > Progress )
   {
      ret = ReadPixel( FgSampler, float2( xy1.x - Progress, xy1.y ) );
   }
   else
   {
      ret = ReadPixel( BgSampler, float2( _bgNormWidth - ( Progress - xy2.x ), xy2.y )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 PushDown( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.y > Progress )
   {
      ret = ReadPixel( FgSampler, float2( xy1.x, xy1.y - Progress ) );
   }
   else
   {
      ret = ReadPixel( BgSampler, float2( xy2.x, 1.0 - ( Progress - xy2.y ) )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 PushLeft( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   // Same as PushRight except going the other way, so :

   float progress = 1.0 - Progress;

   if ( xyNorm.x > progress )
   {
      ret = ReadPixel( BgSampler, float2( xy2.x - progress, xy2.y ) );
   }
   else
   {
      ret = ReadPixel( FgSampler, float2( _bgNormWidth - ( progress - xy1.x ), xy1.y )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 PushUp( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   // Same as PushDown except going the other way, so :

   float progress = 1.0 - Progress;

   if ( xyNorm.y > progress )
   {
      ret = ReadPixel( BgSampler, float2( xy2.x, xy2.y - progress ) );
   }
   else
   {
      ret = ReadPixel( FgSampler, float2( xy1.x, 1.0 - ( progress - xy1.y ) )  );
   }

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Push1 { pass SinglePass { PixelShader = compile PROFILE PushRight(); } }
technique Push2 { pass SinglePass { PixelShader = compile PROFILE PushDown(); } }
technique Push3 { pass SinglePass { PixelShader = compile PROFILE PushLeft(); } }
technique Push4 { pass SinglePass { PixelShader = compile PROFILE PushUp(); } }
