//--------------------------------------------------------------//
// Sharpen
//
// Copyright (c) LWKS Software Ltd.  All Rights Reserved
//--------------------------------------------------------------//
#include "_utils.fx"

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Sharpen";
   string Category    = "Stylize";
   bool CanSize       = true;
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Amount
<
   string Description = "Amount";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.2;

float _OutputWidth;
float _OutputHeight;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
DeclareInput( Input, InputSampler );

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret;

   if ( IsOutOfBounds( xy1 ) )
   {
      ret = kTransparentBlack;
   }
   else
   {
      float xOffset = 1.0 / _OutputWidth;
      float yOffset = 1.0 / _OutputHeight;

      float scaledAmount = Amount * 5.0;
      float fac = scaledAmount / 8.0;

      float4 source = tex2D( InputSampler, xy1 );
      ret = source * ( 1.0 + scaledAmount );

      ret -= tex2D( InputSampler, xy1 + float2( -xOffset, -yOffset ) ) * fac;
      ret -= tex2D( InputSampler, xy1 + float2( 0.0,      -yOffset ) ) * fac;
      ret -= tex2D( InputSampler, xy1 + float2( xOffset,  -yOffset ) ) * fac;
      ret -= tex2D( InputSampler, xy1 + float2( -xOffset, 0.0 ) )      * fac;
      ret -= tex2D( InputSampler, xy1 + float2( xOffset,  0.0 ) )      * fac;
      ret -= tex2D( InputSampler, xy1 + float2( -xOffset,  yOffset ) ) * fac;
      ret -= tex2D( InputSampler, xy1 + float2( 0.0,       yOffset ) ) * fac;
      ret -= tex2D( InputSampler, xy1 + float2( xOffset,   yOffset ) ) * fac;

      ret = lerp( source, ret, source.a );
      ret.a = source.a;
   }

   return ret;
}

technique Sharpen { pass Single_Pass { PixelShader = compile PROFILE ps_main(); } }
